.\" 
.\" file: hoc6.man
.\" 
.\" Date: 17.01.2000 15:50:25 h MET
.\" 
.\" by:   Erich Waelde
.\" 
.\" 06.04.2001 EW
.\" fixed several '\c' into '\\c'
.\" added '#' comment explanation
.\" added i=0 in example, cannot work without
.\" 
.\" 
.\" 
.\" 
.\" 
.\" Header
.\" Proram Section Date
.TH hoc 1 "07.Apr.2001"
.\" 
.\" --- Name -------------------------------------------------------------
.\" 
.SH NAME
hoc \- higher order calculator
.\" 
.\" --- Syntax -----------------------------------------------------------
.\" 
.SH SYNTAX
.I hoc ""

.I hoc 
[file [...] -]
.\" 
.\" --- Description ------------------------------------------------------
.\" 
.SH DESCRIPTION
.I hoc
is a simple programmable interpreter for floating point expressions.
It has C-Style control flow, function definitions and the usual numerical
built-in functions such as cosine and logarithm.
.PP
.I hoc
reads input from files or stdin. A pound character '#' starts a comment the
remainder of the line is ignored. Otherwise it evaluates each expression
and possibly prints the result. To exit
.I hoc,
type and EOF-character, i.e. 'Crtl-d' on Unix, 'Ctrl-z' on VMS.
.PP
.SH CONTENTS
The manpage has the following sections:

.nf
EXPRESSIONS
STATEMENTS AND CONTROL FLOW
INPUT AND OUTPUT: 'read' AND 'print'
FUNCTIONS AND PROCEDURES
EXAMPLES
.fi
.PP
.SH EXPRESSIONS
.I hoc
is an expression language, much like C: although there are serveral
control-flow statements, most statements such as assignments are
expressions whose value is disregarded. For example, the assignment
operator '=' assigns the value of its right hand operand to its left
operand, and yields the value, so multiple assignments work. The
expression grammar is:

.nf
\fBexpr:\fR  number
     | variable
     | ( expr )
     | expr binop expr
     | unop expr
     | function ( arguments )
.fi

.PP
Numbers are floating point. The input format is that recognized by
.B scanf(3): 
an optional sign, digits, decimal point, digits, 'e' or 'E',
an optional sign and the digits of the exponent. At least one digit or
a decimal point must be present; the other components are optional.

.B Variable names
are formed from a letter followed by a string of letters
and numbers.

.B binop 
refers to binary operators such as addition or logical comparison;

.B unop
refers to the logical negation '!' (not) and arithmetic
negation '-' (sign change) operators.
.PP
.B "Table 1: Operators in decreasing precedence"
.PP
.nf
\fB^        \fR exponentiation, right associative
\fB! -      \fR (unary) logical 'not', arithmetic sign change
\fB * /     \fR multiplication, division
\fB+ -      \fR addition, subtraction
\fB> >= < <= == !=\fR
          relational operators greater, greater equal,
          less, less equal, equal, not equal
\fB&&       \fR logical AND (both operands always evaluated)
\fB||       \fR logical OR (both operands always evaluated)
\fB=        \fR assignment, right associative
.fi

.PP
Functions, as described later, may be defined by the user. Function
arguments are expressions separated by commas. There are also a number
of built--in functions, all af which take a single argument:
.PP
.B "Table 2: Built-in Functions"

.nf
\fBabs(x)  \fR  absolute value
\fBatan(x) \fR  arc tangent
\fBcos(x)  \fR  cosine
\fBexp(x)  \fR  exponential
\fBint(x)  \fR  integer part, truncated towards zero
\fBln(x)   \fR  natural logarithm, base E
\fBlog(x)  \fR  logrithm, base 10
\fBsin(x)  \fR  sine
\fBsqrt(x) \fR  square root
.fi

.PP
logical expressions have value 1.0 (true) and 0.0 (false). As in C, any
non-zero value is taken to be true. As is always the case with floating
point numbers, equality comparisons are inherently suspect.

.PP
.B "Table 3: Built-in Constants"

.nf
\fBDEG  \fR 57.2957795130                 180/pi, degrees per radian 
\fBE    \fR  2.7182818284                 base of ln 
\fBGAMMA\fR  0.5772156649                 Euler--Mascheroni constant 
\fBPHI  \fR  1.6180339887                 (sqrt(5)+1)/2, the golden ratio 
\fBPI   \fR  3.1415926535                 pi
\fBCE   \fR  1.606177e-19   Cb            e, electron charge 
\fBME   \fR  9.109389e-31   kg            m_e, electron mass 
\fBMP   \fR  1.672623e-27   kg            m_p, proton mass 
\fBMU   \fR  1.660540e-27   kg            m_u, atomic mass unit 
\fBCC   \fR  2.997925e+08   m/s           c, speed of light 
\fBHH   \fR  6.626075e-34   J s           h, Plancks constant 
\fBEO   \fR  8.854187e-12   F/m           epsilon_0 
\fBMO   \fR  1.256637e-08   N/(A^2)       mu_0 
\fBGN   \fR  6.672598e-11   m^3/(kg s^2)  gamma, gravitational constant 
\fBNA   \fR  6.022136e+23   1/mol         N_L, Avogadro constant 
\fBKK   \fR  1.380658e-23   J/K           k, Boltzmann constant 
\fBSG   \fR  5.670512e-08   W/(m^2 K^4)   sigma, Stefan--Boltzmann constant 
\fBMS   \fR  1.99e30        kg            M_solar, Mass of Sun 
\fBLS   \fR  3.90e26        W             L_solar, Luminosity of Sun 
\fBRS   \fR  6.96e08        m             R_solar, Radius of Sun 
\fBAU   \fR  1.496e11       m             Astr. Unit, distance Sun to Earth
\fBPC   \fR  3.086e16       m             Parsec 
\fBLY   \fR  9.46e15        m             Light year 
.fi
.PP
.\"
.\"
.SH STATEMENTS AND CONTROL FLOW
.PP
.I hoc
statements have the following grammar:
.PP
.nf
\fBstmt:\fR expr
     | variable = expr
     | procedure ( arg-list )
     | while ( expr ) stmt
     | if ( expr ) stmt
     | if ( expr ) stmt else stmt
     | { stmt-list }
     | print expr-list
     | return optional-expr

\fBstmt-list:\fR (nothing)
     | stmt-list stmt
.fi

An assignment is parsed by default as a statement rather than an
expression, so assignments typed interactively do not print their value.

Note that semicolons ';' are not special to
.I hoc:
statements are terminated by newlines. This causes some peculiar behaviour.
The following are legal if-statements:

.nf
if (x<0) print(y) else print(z)

if (x<0) {
    print(y)
}else{
    print(z)
}
.fi

In the second example, the braces ('{','}') are mandatory: the newline
after the 'if' would terminate the statement and produce a syntax
error were the brace omitted.

The syntax and semantics of 
.I hoc
control flow facilities are basically the same as in C. The 'while' and 'if' 
statements are just as in C, except there are no 'break' or 'continue'
statements.


.SH INPUT AND OUTPUT: 'read' AND 'print'

The input function \fBread\fR, like other built-ins, takes a single argument.
Unlike the built-ins, though, the argument is not an expression: it is
the name of a variable. The next number (as defined above) is read from
the standard input and assigned to the named variable. The return value
of \fBread\fR is 1 (true) if a value was read, and 0 (false) if \fBread\fR
encountered end of file or an error.

Output is generated with the \fBprint\fR statement. The arguments to \fBprint\fR
are a comma-separated list of expressions and strings in double quotes
("..."), as in C. Newlines must be supplied; they are never provided
automatically by \fBprint\fR.

Note that \fBread\fR is a special built-in function, and therefore takes a
single parenthesized argument, while \fBprint\fR is statement that takes
a comma-separated, unparenthesized list:

.nf
while (read(x)){
    print "value is ", x, "\\n"
}
.fi
.PP
.\"
.\"
.SH FUNCTIONS AND PROCEDURES

Functions and procedures are distinct in 
.I hoc,
although they are defined
by the same mechanism. This distinction is simply for run-time error
checking: it is an error for a procedure to return a value, and for a
function not to return one. The definition syntax is:
.nf
function:       func name() stmt
procedure:      proc name() stmt
.fi
\fBname\fR may be the name of any variable --- built-in functions are excluded.
The definition, up to the opening brace or statement, must be on one line,
as with the 'if' statements above.

Unlike C, the body of a function or procedure may be any statement, not
necessarily a compound (brace-enclosed) statement. Since semicolons have
no meaning in 
.I hoc,
a null procedure is performed by an empty pair of braces.

Functions and procedures may take arguments, separated by commas, when
invoked. Arguments are reffered to as in the shell: '$3' refers to the
third (1-indexed) argument. They are passed by value and within functions
are semantically equivalent to variables. It is an error to refer to an
argument numbered greater than the number of arguments passed to the
routine. The error checking is done dynamically, however, so a routine
may have variable number of arguments if initial arguments effect the
number of arguments to be referenced (as in C's 'printf').

Functions and procedures may recurse, but the stack has limited depth
(about 100 calls). The following shows a 
.I hoc
\ definition of Ackermann's
function:
.nf
shell> hoc
        H O C  --   higher order calculator  --  version 6
hoc> func ack() {
    if($1==0) return $2+1
    if($2==0) return ack($1-1,1)
    return ack($1-1, ack($1, $2-1))
}
ack(3,2)
-->     29
hoc> ack(3,3)
-->     61
hoc> ack(3,4)
hoc>  stack overflow near line 9
...
.fi


.SH EXAMPLES
.IP "" 5

1. Stirling's formula:

n!  approx.  sqrt(2npi)*(n/E)^n*(1+1/(12n))

.nf
...
hoc> func stirl(){
    return sqrt(2*$1*PI) * ($1/E)^$1 * (1+1/(12*$1))
}
stirl(10)
-->     3628684.7
hoc> stirl(20)
-->     2.43288179e+18
...
.fi

2. Factorial function:

.nf
...
hoc> func fac() if ($1<=0) return 1 else return $1*fac($1-1)
fac(4)
-->     24
hoc> fac(20)
-->     2.43290201e+18
...
.fi

3. Ratio of factorial to Stirling approximation:

.nf
...
hoc> i=0
hoc> while((i=i+1)<=20){
        print i,"\\t",fac(i)/stirl(i),"\\n"
}
1       1.0010193
2       1.0005188
3       1.000279
4       1.0001714
5       1.0001154
6       1.0000828
7       1.0000623
8       1.0000485
9       1.0000388
10      1.0000318
11      1.0000265
12      1.0000224
13      1.0000192
14      1.0000166
15      1.0000146
16      1.0000128
17      1.0000114
18      1.0000102
19      1.0000092
20      1.0000083
...
.fi
.\" 
.\" ----------------------------------------------------------------------
.\" 
.PP
.\" 
.\" --- See Also ---------------------------------------------------------
.\" 
.SH "SEE ALSO"
bc(1),
dc(1),
scanf(3)
.\" 
.\" --- Diagnostics ------------------------------------------------------
.\" 
.\" 
.\" --- Author -----------------------------------------------------------
.\" 
.SH AUTHOR
.I hoc 
and the man page have been written by Erich Waelde (Erich_Waelde@agilent.com).
.\"
.\" --- Disclaimer -------------------------------------------------------
.\"
.SH DISCLAIMER
\fBThis tool is HellWare!!! Use at your own risk!!!\fR
If you ask silly questions about it, then you and your data shall go
straight to hell without further notice.
.\"
.\" --- History ----------------------------------------------------------
.\"
.SH HISTORY
.I hoc
has been coded along the example in Brian W. Kernighan and Rob Pike's
book
.I The Unix Progamming Environment
(Prentice-Hall 1984). This chapter is IMHO one of the best worked examples
of program design using a clear cut grammar and of program development
utilizing powerful tools where appropriate. Contrast the first version of
hoc in the chapter metioned above against the pocket calculator example
from Kernighan and Ritchies classic
.I The C Programming Language
(2nd ed. Prentice-Hall, 1992). I found it amazing how simple and elegant
the calculator becomes with the use of a short grammar and yacc. Read the
books and enjoy.






