/* hoc6: EW 01.09.93 */

%{
#undef yywrap
#include "hoc6.h"
#define code2(c1,c2)     code(c1); code(c2)
#define code3(c1,c2,c3)  code(c1); code(c2); code(c3)

int      indef;
%}

%union 
{                         /* Stacktype, wahlweise  */
    Symbol *sym;          /* pointer to symbol table */
    Inst   *inst;         /* pointer to machine instruction */
    int     narg;         /* number of arguments */
}

%token <sym>   NUMBER STRING PRINT VAR BLTIN UNDEF WHILE IF ELSE
%token <sym>   FUNCTION PROCEDURE RETURN FUNC PROC READ
%token <narg>  ARG
%type  <inst>  expr stmt asgn prlist stmtlist 
%type  <inst>  cond while if begin end
%type  <sym>   procname
%type  <narg>  arglist
%right '='
%left  OR
%left  AND
%left  GT GE LT LE EQ NE
%left  '+' '-'
%left  '*' '/' '%'
%left  UNARYMINUS NOT
%right '^'
%%
list:                      /* leere Liste */
      | list '\n'          
      | list defn '\n'          
      | list asgn '\n'     { code(STOP); return 1; }
      | list stmt '\n'     { code(STOP); return 1; }
      | list expr '\n'     { code2(print, STOP); return 1; }
      | list error '\n'    { yyerrok; }
      ;
asgn:   VAR '=' expr       { code3(varpush, (Inst)$1, assign); $$=$3; }
      | ARG '=' expr       { defnonly("$"); code2(argassign, (Inst)$1); $$=$3;}
      ;
stmt:   expr               { code(pop); }
      | RETURN             { defnonly("return"); code(procret); }
      | RETURN expr        { defnonly("return"); $$=$2; code(funcret); }
      | PROCEDURE begin '(' arglist ')'
                           { $$=$2; code3(call, (Inst)$1, (Inst)$4); }
      | PRINT prlist       { $$=$2; }
      | while cond stmt end {
                             ($1)[1] = (Inst)$3;     /* body of loop */
                             ($1)[2] = (Inst)$4; }   /* end, if cond fails */
      | if cond stmt end   {                     /* without else-block */
                             ($1)[1] = (Inst)$3;     /* then-part */
                             ($1)[3] = (Inst)$4; }   /* end, if cond fails */
      | if cond stmt end ELSE stmt end {         /* with else-block */
                             ($1)[1] = (Inst)$3;     /* then-part */
                             ($1)[2] = (Inst)$6;     /* else-part */
                             ($1)[3] = (Inst)$7; }   /* end if-then-else finished */
      | '{' stmtlist '}'   { $$=$2; }
      ;
cond:   '(' expr ')'       { code(STOP); $$=$2; }
      ;
while:  WHILE              { $$=code3(whilecode, STOP, STOP); }
      ;
if:     IF                 { $$=code(ifcode); code3(STOP, STOP, STOP); }
      ;
begin:                     { $$=progp; }
      ;
end:                       { code(STOP); $$=progp; }
      ;
stmtlist:                  { $$=progp; }
      | stmtlist '\n'
      | stmtlist stmt 
      ;
expr:   NUMBER             { $$=code2(constpush, (Inst)$1); }
      | VAR                { $$=code3(varpush, (Inst)$1, eval); }
      | ARG                { defnonly("$"); $$=code2(arg, (Inst)$1); }
      | asgn
      | FUNCTION begin '(' arglist ')'
                           { $$=$2; code3(call, (Inst)$1, (Inst)$4); }
      | READ '(' VAR ')'   { $$=code2(varread, (Inst)$3); }
      | BLTIN '(' expr ')' { $$=$3; code2(bltin, (Inst)$1->u.ptr); }
      | '(' expr ')'       { $$=$2; }
      | expr '+' expr      { code(add); }
      | expr '-' expr      { code(sub); }
      | expr '*' expr      { code(mul); }
      | expr '/' expr      { code(ddiv); }
      | expr '^' expr      { code(power); }
      | expr '%' expr      { code(mod); }
      | '-' expr %prec UNARYMINUS
                           { $$=$2; code(negate); }
      | expr GT expr       { code(gt); }
      | expr GE expr       { code(ge); }
      | expr LT expr       { code(lt); }
      | expr LE expr       { code(le); }
      | expr EQ expr       { code(eq); }
      | expr NE expr       { code(ne); }
      | expr AND expr      { code(and); }
      | expr OR expr       { code(or); }
      | NOT expr           { $$=$2; code(not); }
      ;
prlist: expr               { code(prexpr); }
      | STRING             { $$=code2(prstr, (Inst)$1); }
      | prlist ',' expr    { code(prexpr); }
      | prlist ',' STRING  { code2(prstr, (Inst)$3); }
      ;
defn:   FUNC procname      { $2->type=FUNCTION; indef=1; }
        '(' ')' stmt       { code(procret); define($2); indef=0; }
      | PROC procname      { $2->type=PROCEDURE; indef=1; }
        '(' ')' stmt       { code(procret); define($2); indef=0; }
      ;
procname: VAR
      | FUNCTION
      | PROCEDURE
      ;
arglist:                   { $$=0; }
      | expr               { $$=1; }
      | arglist ',' expr   { $$=$1+1; }
      ;
%%
    /* Ende Grammatik */

#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>

char     progname[]="hoc6";
int      lineno = 1;
jmp_buf  begin;
char     *infile;
extern FILE *yyin;
char     **gargv;
int      gargc;
int      c;


char *dq_backslash(char *s)
{
    /* scan the string, if '\''x' is found, replace by '\x' if 'x' is found
       in transtab. Since the sting will only become shorter this way, copy
       the sting onto itself */

    static char transtab[]="b\bf\fn\nr\rt\t";
    char *a, *b, *c;
    char *index();

    a=s,b=s;
    while (a[0]!='\0')
    {
        /* skip quotes */
        if (a[0] == '"')
        {
            a++;
            continue;
        }
        if (a[0] == '\\' && islower(a[1]) && (c=index(transtab,a[1])))
        {   
            *b=c[1];
        	a++;
        }
        else
        {   
            *b = *a;
        }
        a++;
        b++;
    }
    *b='\0';
    return s;
}

defnonly(char *s)
{   void execerror(char *s, char *t);
    if(!indef)
        execerror(s, "used outside definition");
}

yyerror(char *s)
{
    void warning(char *s, char *t);

    warning(s, (char *)0);
}

void execerror(char *s, char *t)
{
    void warning(char *s, char *t);
    
    warning(s,t);
    /*fseek(fin,0L,2); /* flush rest of file */
    fseek(yyin,0L,2); /* flush rest of file */
    longjmp(begin,0);
}

void fpecatch(void)
{
    void execerror(char *s, char *t);

    execerror("floating point exception", (char *)0);

    /*return 1;*/
}

main(int argc, char **argv)
{
    int     i;
    void    fpecatch(void);
    void    run(void);

    /*progname = argv[0]; /* sieht auf der VAX schrecklich aus */

    if(argc==1)  /* fake an argument list */
    {   static char *stdinonly[] = { "-" };
        gargv = stdinonly;
        gargc = 1;
    }
    else
    {   gargv = argv + 1;
        gargc = argc - 1;
    }
    init();
    while(yywrap() == 0)
        run();

    return 0;
}

int yywrap(void)
{
    if(gargc-- <= 0)
    {   /*fprintf(stderr,"yywrap: no more arguments\n");*/
        return 1;
    }
    if (yyin && yyin != stdin)
    {   /*fprintf(stderr,"yywrap: closing current file %s.\n",infile);*/
        fclose(yyin);
    }
    infile = *gargv++;
    lineno = 1;
    if(strcmp(infile, "-")==0)
    {   yyin = stdin,
        infile = 0;
    /*fprintf(stderr,"yywrap: read from stdin\n");*/
    }
    else if ((yyin=fopen(infile,"r"))==NULL)
    {   fprintf(stderr, "%s: cannot open file '%s'\n",progname,infile);
            return yywrap();
    }
    else
    {
        /*fprintf(stderr,"yywrap: opening file %s\n",infile);*/
        ;
    }
    return 0; /* more input waiting at yyin */
}

void run(void)
{   void fpecatch(void);

    setjmp(begin);
    signal(SIGFPE, fpecatch); 
    
    for( initcode(); yyparse(); initcode())
        execute(progbase);
}

void warning(char *s, char *t)
{
    fprintf(stderr,"%s: %s",progname,s);
    if(t)
        fprintf(stderr," %s",t);
    if(infile)
        fprintf(stderr," in file '%s'", infile);
    fprintf(stderr," near line %d\n",lineno);
    while(c!='\n' && c!= EOF)
    {
        /*c = getc(fin); /* flush rest of input line */
        c = getc(yyin); /* flush rest of input line */
    }
    if(c=='\n')
        lineno++;
}


