/* init.c */

#include "hoc6.h"
#include "y.tab.h"
#include <math.h>
#include <stdio.h>

extern double Log(double x);
extern double Log10(double x);
extern double Exp(double x);
extern double Sqrt(double x);
extern double Int(double x);

static struct 
{   char *name;
    int  kval;
} keywords[] =
{
    "proc",    PROC,
    "func",    FUNC,
    "return",  RETURN,
    "if",      IF,
    "else",    ELSE,
    "while",   WHILE,
    "print",   PRINT,
    "read",    READ,
    0,         0
};

static struct
{   char    *name;
    double  cval;
} consts[] = 
{
    "PI",	3.14159265358979323846,
    "E",	2.71828182845904523536,
    "GAMMA",	0.57721566490153286060,
    "DEG",     57.29577951308232087680,
    "PHI",	1.61803398874989484820,
    "CE" ,  1.606177e-19,
    "ME" ,  9.109389e-31,
    "MP" ,  1.672623e-27,
    "MU" ,  1.660540e-27,
    "CC" ,  2.997925e+08,
    "HH" ,  6.626075e-34,
    "EO" ,  8.854187e-12,
    "MO" ,  1.256637e-08,
    "GN" ,  6.672598e-11,
    "NA" ,  6.022136e+23,
    "KK" ,  1.380658e-23,
    "SG" ,  5.670512e-08,
    "MS" ,  1.990e30,
    "LS" ,  3.900e26,
    "RS" ,  6.960e08,
    "AU" ,  1.496e11,
    "PC" ,  3.086e16,
    "LY" ,  9.460e15,
    0,		0
};

static struct 
{   char    *name;
    double  (*func)();
} builtins[] =
{
    "sin",	sin,
    "cos",	cos,
    "atan",	atan,
    "ln",	Log,
    "log",	Log10,
    "exp",	Exp,
    "sqrt",	Sqrt,
    "int",	Int,
    "abs",	fabs,
    0,		0
};


void init(void)
{   int		i;
    Symbol	*s;

    for(i=0; keywords[i].name; i++)
        install(keywords[i].name,keywords[i].kval, 0.0);

    for(i=0; consts[i].name; i++)
	install(consts[i].name, VAR, consts[i].cval);

    for(i=0; builtins[i].name; i++)
    {	s = install(builtins[i].name, BLTIN, 0.0);
		s->u.ptr = builtins[i].func;
    }

    fprintf(stdout,"\tH O C  --   higher order calculator  --  version 6.03\n");
    /*
    fprintf(stdout,"\tfor help read sat1$dka500:[user.ai22.tools.hoc]hoc.1\n");
   fprintf(stdout,"\tto exit type \"EOF\" -- CTRL-Z on VAX, CTRL-D on UNIX\n");
    */
}

